<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once("PolarisLinkSite.php");	
	require_once("PolarisLinkCacheManager.php");
	require_once("PolarisLinkRefCache.php");

	class PolarisLinkLogCtx
	{
		private $linkId;		
		private $entity = null;
		private $msg = null;
		private $changes = [];

		public function __construct($linkID, $entity, $msg, $args = null)
		{
			$this->linkID = $linkID;
			$this->entity = $entity;
			$this->msg = $args ? vsprintf($msg, $args) : $msg;
		}

		public function LogChange($property, $value, $oldValue = null)
		{			
			$this->changes[] = $oldValue ? 
									sprintf("%s : %s → %s", $property, $oldValue, $value) :
									sprintf("%s → %s", $property, $value);

			return $this;
		}

		public function LogCreate()
		{
			$this->changes[] = "";

			return $this;
		}

		/**
		 * Procède à la sauvegarde de l'entité s'il y a un journal
		 * et renvoi vrai si l'entité a été sauvée
		 */
		public function Commit()
		{
			if (!empty($this->changes))
			{
				if (!empty($this->entity))
					$this->entity->save();
				$this->Flush();
				return true;
			}
			return false;
		}

		public function Flush()
		{
			$changesStr = join(", ", $this->changes);

			PolarisLink::Log(LOG_INFO, $this->msg . (empty($changesStr) ? "" : " : " . $changesStr), $this->linkID, null, $this->entity, true);

			$this->changes = [];
		}

		public function Dump()
		{
			foreach ($this->changes as $change)
				var_dump($this->msg . (empty($change) ? "" : " : " . $change));
		}
	}

	abstract class PolarisLinkImportBase 
	{
		public $DebugMode = false;
		protected $JobManager = null;
		protected $Link = null;
		protected $DB = null;		
		protected $RefsCache = null;
		protected $Job;
		
		protected function setCache(PolarisLinkCache $cache)
		{
			$this->JobManager->setCache($cache);
		}
		
		protected function getCache($type)
		{
			return $this->JobManager->getCache($type);
		}

		protected function NewLogCtx($entity, $msg, $args = null)
		{
			return new PolarisLinkLogCtx($this->Link->LinkID, $entity, $msg, $args);
		}

		public function Log($entity, $msg, $args = null)
		{
			$this->NewLogCtx($entity, $msg, $args)->LogCreate()->Commit();
		}

		/*
			Requête SQL sécurisée
		*/
		protected function SafeSQL($query)
		{
			$results = $this->DB->executeS($query, true, false);
			if ($results === false)
				throw new PrestaShopDatabaseException($this->DB->getMsgError());
			return $results;			
		}
		
		/*
			On journalise l'erreur
		*/
		protected function logError($ex)		
		{
			$this->JobManager->logError($ex);
		}		
		
		protected function SetShopContext($opt)
		{
			if ($opt == null || $opt == 0 || !Shop::isFeatureActive())
				return;

			if (strlen($opt) > 0 && $opt[0] == '*')
				Shop::setContext(Shop::CONTEXT_GROUP, substr($opt, 1));
			else
				Shop::setContext(Shop::CONTEXT_SHOP, $opt);			
		}
		
		protected function SetCatalogContext()
		{
			$this->SetShopContext($this->Link->CatalogShopID);
		}
		
		protected function SetPricesContext()
		{
			$this->SetShopContext($this->Link->PricesShopID);
		}
		
		/*
			Obtient la valeur de la balise ou de l'attribut, émet une erreur si non spécifiée
		*/
		protected function GetMandatory($data, $xml_node_name, $attribute_name = null)
		{
			if (empty($attribute_name))
			{
				if (!isset($data->$xml_node_name) || (string) $data->$xml_node_name == null || (string) $data->$xml_node_name == "")
					throw new Exception("la balise ".$xml_node_name." n'a pas été spécifiée alors qu'attendue");
				
				return $data->$xml_node_name;
			}
			else
			{
				$value = $data;
				if (!empty($xml_node_name))
					$value = $data->$xml_node_name;
				
				if (!isset($value[$attribute_name]) || empty($value[$attribute_name]))
					throw new Exception("l'attribut ".$xml_node_name."[".$attribute_name."] n'a pas été spécifiée alors qu'attendu");
				
				return $value[$attribute_name];
			}
		}
		
		/*
			Obtient la valeur de la balise ou de l'attribut, renvoi la valeur par défaut si non spécifiée
		*/
		protected function GetOptional($data, $xml_node_name, $attribute_name = null, $defaultValue = null)
		{
			if (empty($attribute_name))
			{
				if (!isset($data->$xml_node_name) || (string) $data->$xml_node_name == null || (string) $data->$xml_node_name == "")
					return $defaultValue;
				return $data->$xml_node_name;
			}
			else
			{
				$value = $data;
				if (!empty($xml_node_name))
					$value = $data->$xml_node_name;
				
				if (!isset($value[$attribute_name]) || empty($value[$attribute_name]))
					return $defaultValue;
				
				return $value[$attribute_name];
			}
		}
		
		/*
			Obtient un tableau contenant toutes les langues renseignées dans la valeur value
		*/
		protected function getLgValue($value, $checkFunc = null)
		{
			$values = array();

			if ($value != null && count($value->children()) > 0)
			{
				foreach ($value->children() as $langXmlId => $child) 
				{		
					$lang = $this->JobManager->getLangFor($langXmlId);					
					if ($lang != NULL)					
						$values[$lang["id_lang"]] = (string) $child;
				}
			}
			else
				$values[$this->Link->LangID] = (string) $value;

			foreach ($values as $id_lang => $value)
			{
				if ($checkFunc != null)
					$values[$id_lang] = $checkFunc($values[$id_lang]);
			}			

			return $values;
		}
		
		/*
			Complète les langues non renseignées dans le champs par sa définition de la passerelle, ou si absent, par la langue par défaut
			du système, ou par défaut, toute autre langue renseignée. 
			Si aucune langue n'est renseignée, prends la valeur par défaut.
		*/
		protected function AutoFillEmptyLangValue($field, $defaultValue = null)
		{
			$fillValue = null;

			if ($field === null && $defaultValue !== null)
			{
				$field = array();
				foreach (Language::getIDs(false) as $id_lang)
				{
					if (!isset($field[$id_lang]))
						$field[$id_lang] = $defaultValue;
				}

				return $field;
			}

			if (isset($field[$this->Link->LangID]) && $field[$this->Link->LangID] !== null)
				$fillValue = $field[$this->Link->LangID];

			if ($fillValue === null)
			{
				$defaultLang = Configuration::get("PS_DEFAULT_LANG");
				if (isset($field[$defaultLang]) && $field[$defaultLang] !== null)
					$fillValue = $field[$this->Link->LangID];

				if ($fillValue === null)
				{
					// toujours rien ? On prend le premier qui vient
					foreach ($field as $k => $v)
					{
						if ($v !== null)
						{
							$fillValue = $v;
							break;
						}
					}
				}
			}

			// Ok, maintenant, tour de table des langues !
			foreach (Language::getIDs(false) as $id_lang)
			{
				if (!isset($field[$id_lang]))
					$field[$id_lang] = $fillValue;
			}			
			
			return $field;
		}

		protected function BuildLinkRewrite($value)
		{
			if (is_array($value))
			{
				$prop = array();
				foreach ($value as $k => $v)
					$prop[$k] = $this->BuildLinkRewrite($v);				
				return $prop;
			}
			else
			{
				$prop = trim(Tools::link_rewrite($value));
				if (empty($prop))
					$prop = "link";
				return $prop;
			}			
		}
		
		/**
		 * Formats a Prestashop value
		 *
		 * @param mixed	$value
		 * @param int	$type
		 * @param bool	$with_quotes
		 * @param bool	$purify
		 * @param bool	$allow_null
		 * @return mixed
		 */
		public static function formatValue($value, $type, $purify = true, $allow_null = false)
		{
			if ($allow_null && $value === null) {
				return null;
			}

			switch ($type) {
				case ObjectModel::TYPE_INT:
					return (int)$value;

				case ObjectModel::TYPE_BOOL:
					return (int)$value;

				case ObjectModel::TYPE_FLOAT:
					return (float)str_replace(',', '.', $value);

				case ObjectModel::TYPE_DATE:
					if (!$value) {
						return '0000-00-00';
					}

					return $value;

				case ObjectModel::TYPE_HTML:
					if ($purify) {
						$value = Tools::purifyHTML($value);
					}
					return $value;

				case ObjectModel::TYPE_SQL:
					return $value;

				case ObjectModel::TYPE_NOTHING:
					return $value;

				case ObjectModel::TYPE_STRING:
				default :
					return $value;
			}
		}
		
		/*
			Copie une propriété optionnelle pour la pousser dans l'instance Prestashop $obj si elle existe dans le XML
			Possibilité de définir une valeur par défaut si rien n'est trouvé dans le XML
			Exécute la fonction $checkFunc pour vérifier la valeur

			Renvoi vrai si la propriété a été modifiée
		*/
		protected function updOptProperty($log, $obj, $property, $data, $value, $default = null, $checkFunc = null)
		{
			if (isset($data->$value))
			{
				// La donnée existe dans le XML, cas simple
				return $this->updProperty($log, $obj, $property, $data->$value, $checkFunc);
			}
			// La donnée n'existe pas, avons nous une valeur par défaut ?
			if ($default)
				return $this->updProperty($log, $obj, $property, $default, $checkFunc);
			
			// Si on arrive là, alors on fait rien !
		}
		
		/*
			Copie une propriété présente dans $value pour la pousser
			dans l'instance Prestashop $obj.

			Exécute la fonction $checkFunc pour vérifier la valeur

			Renvoi vrai si la propriété a été modifiée
		*/		
		protected function updProperty($log, $obj, $property, $value, $checkFunc = null, $langId = null, $completeAllLangs = false)
		{
			// Cherchons la définition du champs
			$fieldDef = get_class($obj)::$definition['fields'][$property];

			if (!$langId)
			{
				if (isset($fieldDef["lang"]) && $fieldDef["lang"])				
				{
					$changes = false;

					// C'est une propriété langue					
					
					// A priori, on communique avec des balises de langues
					if ($value != null)
					{
						$langValues = [];

						if (count($value) > 0)
						{
							if (is_array($value))
							{
								$langValues = $value;
							}
							else if (($value instanceof \SimpleXMLElement) && $value->count())
							{
								foreach ($value->children() as $langXmlId => $child)
								{		
									$l = $this->JobManager->getLangFor($langXmlId);	
									if ($l != NULL)
										$langValues[$l["id_lang"]] = (string) $child;
								}
							}
						}

						if (count($langValues) == 0)
							$langValues[$this->Link->LangID] = (string) $value;

						// On s'assure de modifier toutes les langues
						if ($completeAllLangs || (isset($fieldDef["required"]) && $fieldDef["required"]))
						{
							$currents = $obj->$property;
							$currentValueLang = isset($currents[$this->Link->LangID]) ? $currents[$this->Link->LangID] : "";
							$newValueLang = isset($langValues[$this->Link->LangID]) ? $langValues[$this->Link->LangID] : "";
							foreach (Language::getIDs(false) as $id_lang)
							{
								if (!isset($langValues[$id_lang]) && (empty($currents[$id_lang]) || $currents[$id_lang] == $currentValueLang))
									$langValues[$id_lang] = $newValueLang;
							}
						}

						foreach ($langValues as $id_lang => $value)
							$changes |= $this->updProperty($log, $obj, $property, $value, $checkFunc, $id_lang);
					}

					return $changes;
				}				
			}
			
			$value = (string) $value;
			
			// Normalisation, validation à coup sûr !
			if ($checkFunc != null)
				$value = $checkFunc($value);

			// Formattage prestashop !
			$value = self::formatValue(
				$value, 
				$fieldDef['type'], 
				(isset($fieldDef['validate']) && Tools::strtolower($fieldDef['validate']) == 'iscleanhtml'), 
				!empty($fieldDef['allow_null']));

			if (!$langId)
			{
				// Champ normal
				if ($obj->$property != $value)
				{
					$log->LogChange($property, $value, $obj->$property);
					$obj->$property = $value;					
					return true;
				}
			}
			else
			{
				// Champ de langue
				if (!isset($obj->$property[$langId]) || $obj->$property[$langId] != $value)
				{
					$l = $this->JobManager->getLangFor("ps".$langId);	
					$l = $l ? $l["language_code"] : "??";

					$log->LogChange($property." (".$l.")", $value, isset($obj->$property[$langId]) ? $obj->$property[$langId] : null);
					
					$obj->$property[$langId] = $value;

					return true;
				}
			}

			return false;
		}

		/**
		 * Update une propriété et sauve immédiatement
		 */
		protected function oneTimeUpdProperty($msg, $obj, $property, $value, $checkFunc = null, $langId = null)
		{
			$log = $this->NewLogCtx($obj, $msg);
			$this->updProperty($log, $obj, $property, $value, $checkFunc, $langId);
			$log->Commit();
		}
		
		protected function GetDateTime($value)
		{
			return $value;
		}		
		
		/*
			Renvoi le premier argument non vide
		*/
		protected function FirstNonEmpty($arg1, $arg2 = null, $arg3 = null, $arg4 = null, $arg5 = null, $arg6 = null)
		{
			if (!empty($arg1) && !empty(trim($arg1))) return $arg1;
			if (!empty($arg2) && !empty(trim($arg2))) return $arg2;
			if (!empty($arg3) && !empty(trim($arg3))) return $arg3;
			if (!empty($arg4) && !empty(trim($arg4))) return $arg4;
			if (!empty($arg5) && !empty(trim($arg5))) return $arg5;
			if (!empty($arg6) && !empty(trim($arg6))) return $arg6;
			
			return null;
		}
		
		/*
			Renvoi le nombre de combinaison d'un produit, ou 0 si ce dernier n'existe pas
		*/
		protected function CountProductNbCombinations($product_id)
		{
			$sql = "SELECT COUNT(id_product_attribute) AS c
					FROM " . _DB_PREFIX_ . "product_attribute
					WHERE id_product = ".$product_id." AND id_product_attribute <> 0";
					
			if ($rows = $this->SafeSQL($sql)) 
			{
				foreach ($rows as $row)
					return $row["c"];
				return 0;
			}
			
			return NULL;
		}
		
		protected function GetProduct($id_product)
		{
			if (empty($id_product))
				return NULL;
			
			if (is_array($id_product))
				$id_product = $id_product["id_product"];
			
			$product = new Product($id_product);
			if ((int) $product->id == (int) $id_product)
				return $product;
			return NULL;
		}		
		
		protected function GetFirstReferencesFromList($list)
		{
			return $this->GetReferencesFromList($list, true);
		}
		
		protected function GetReferencesFromList($list, $onlyFirst = false)
		{
			if (empty($list))
				return $onlyFirst ? NULL : array();
			
			$ids = array();
			foreach (explode(',', $list) as $e)
			{
				$id = trim($e);
				if (!empty($id))
				{
					if ($onlyFirst)
						return $id;
					$ids[] = $id;
				}
			}
			
			if ($onlyFirst)
				return count($ids) > 0 ? $ids[0] : NULL;

			return $ids;
		}

		/*
			Recherche un produit et sa combinaison prestashop correspondant à la référence Polaris, ou au code barre gencode, ou 
			null si non trouvé et pas d'exception demandée.
			$lazyProduct = si vrai, ne s'assure pas que le produit trouvé est dépourvu de combinaisons si le code trouvé est celui du produit et non d'une combinaison
		*/
		public function GetProductCombinationFor($referencesList, 
							$eansListe = NULL, 
							$exceptionIfNotFound = false,
							$searchInPolarisBackup = false,
							$lazyProduct = false
							)
		{
			// Avec ou sans cache préchargé ?
			if ($this->RefsCache != null)
			{
				$references = array();
				foreach (explode(',', $referencesList) as $reference)
				{
					$elt = trim($reference);
					if (!empty($reference))
						$references[] = $reference;
				}
				
				$eans = array();
				if ($eansListe)
				{
					foreach (explode(',', $eansListe) as $ean)
					{
						$elt = trim($ean);
						if (!empty($ean))
							$eans[] = $ean;
					}
				}

				if ($row = $this->RefsCache->get($references, $eans, $searchInPolarisBackup))
					return $row;				
			}
			else
			{
				// Pas de cache ?
				$suppliers = $this->Link->SupplierID;
				if ($searchInPolarisBackup)
					$suppliers .= ", ".$this->Link->SupplierBackupID;
			
				$references = array();
				foreach (explode(',', $referencesList) as $reference)
				{
					$elt = trim($reference);
					if (!empty($reference))
						$references[] = "'".pSQL($reference)."'";
				}
				
				$eans = array();
				foreach (explode(',', $eansListe) as $ean)
				{
					$elt = trim($ean);
					if (!empty($ean))
						$eans[] = "'".pSQL($ean)."'";
				}

				// On recherche le produit
				if (count($references) > 0)
				{
					$sql = "SELECT s.id_product, s.id_product_attribute
							FROM " . _DB_PREFIX_ . "product_supplier AS s
							LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = s.id_product)
							LEFT JOIN " . _DB_PREFIX_ . "product_attribute AS pa ON (pa.id_product = s.id_product AND pa.id_product_attribute = s.id_product_attribute)
							WHERE s.id_supplier IN (".$suppliers.") AND s.product_supplier_reference IN (".implode(',', $references).") AND s.id_product_attribute <> 0
								AND p.id_product IS NOT NULL
								AND pa.id_product_attribute IS NOT NULL
							ORDER BY s.id_product ASC, s.id_product_attribute ASC";
					if ($rows = $this->SafeSQL($sql)) 
					{
						foreach ($rows as $row)
							return $row;
					}
				}
				
				if (count($eans) > 0)
				{
					// Non trouvé ? On essaye avec le code EAN (gencode) !
					$sql = "SELECT pa.id_product, pa.id_product_attribute
							FROM " . _DB_PREFIX_ . "product_attribute AS pa
								LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = pa.id_product)
							WHERE pa.ean13 IN (".implode(',', $eans).") 
								AND p.id_product IS NOT NULL
							ORDER BY pa.id_product ASC, pa.id_product_attribute ASC";
					if ($rows = $this->SafeSQL($sql)) 
					{
						foreach ($rows as $row)
							return $row;
					}
				}
				
				if (count($references) > 0)
				{
					// Ok... Peut-être que c'est un produit SANS combinaison,
					// du coup, on importe que si le produit est SANS combinaison			
					$sql = "SELECT s.id_product, 0 AS id_product_attribute
							FROM " . _DB_PREFIX_ . "product_supplier AS s
							LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = s.id_product)
							WHERE s.id_supplier IN (".$suppliers.") AND product_supplier_reference IN (".implode(',', $references).") 
								AND p.id_product IS NOT NULL
							ORDER BY id_product ASC";
							
					if ($rows = $this->SafeSQL($sql)) 
					{
						foreach ($rows as $row)
						{
							if ($lazyProduct || $this->CountProductNbCombinations($row["id_product"]) == 0)
								return $row;
						}
					}
				}
				
				if (count($eans) > 0)
				{
					// Non trouvé ? On essaye avec le code EAN (gencode) sans combinaison !
					$sql = "SELECT p.id_product, 0 AS id_product_attribute
							FROM " . _DB_PREFIX_ . "product AS p							
							WHERE p.ean13 IN (".implode(',', $eans).") 
							ORDER BY p.id_product ASC";
					if ($rows = $this->SafeSQL($sql)) 
					{
						foreach ($rows as $row)
							if ($lazyProduct || $this->CountProductNbCombinations($row["id_product"]) == 0)
								return $row;
					}
				}
			}
			
			// Non trouvé ?
			if ($exceptionIfNotFound)
			{
				if (!empty($referencesList) && !empty($eansListe))
					throw new Exception(sprintf("produit ref %s ou ean %s non trouvé", $referencesList, $ean));
				if (!empty($referencesList))
					throw new Exception(sprintf("produit ref %s non trouvé", $referencesList));
				if (!empty($eansListe))
					throw new Exception(sprintf("produit ean %s non trouvé", $eansListe));
				throw new Exception(sprintf("produit non trouvé"));
			}
			
			return NULL;
		}
		
		/*
			Renvoi la photo Polaris pour une photo du répertoire
			si la photo du répertoire n'est pas une photo Polaris, renvoi -1
			sinon renvoi NULL si aucune photo
		*/
		protected function GetPolarisPhotoFor($filename)
		{
			if (!file_exists($filename))
				return NULL;
			
			if (!is_link($filename))
			{
				// Recherche de notre indice ..
				if (file_exists($filename.".polarislink"))
					return file_get_contents($filename.".polarislink");
				return NULL;
			}
			
			// Lecture de la destination
			return basename(readlink($filename), ".jpg");
		}		

		/*
		* Reconstruction des miniatures
		*/
		protected function RebuildThumbnails($thumbnailTypes, $miniName, $imagesPath, $objectId, $imageId, $imageFormat = 'jpg')
		{
			$res = true;
			$generate_hight_dpi_images = (bool)Configuration::get('PS_HIGHT_DPI');

			/* Génération des thumbnails */
			foreach (ImageType::getImagesTypes($thumbnailTypes) as $k => $image_type) 
			{
				$src = $imagesPath . $imageId . '.' . $imageFormat;				
				$dst = $imagesPath . $imageId . '-' . stripslashes($image_type['name']) . '.' . $imageFormat;

				// Nettoyage
				foreach (glob($imagesPath . $imageId . '-' . stripslashes($image_type['name']) . '.*', GLOB_NOSORT) as $file)
					@unlink($file);

				if (!ImageManager::resize($src, $dst, (int)$image_type['width'], (int)$image_type['height']))
				{
					$this->logError(sprintf("Impossible de générer le thumbnail %s", $dst));
					$res = false;
				}

				if ($generate_hight_dpi_images) 
				{
					$dst = $imagesPath . $imageId . '-' . stripslashes($image_type['name']) . '2x.' . $imageFormat;
					@unlink($dst);
					if (!ImageManager::resize($src, $dst, (int)$image_type['width']*2, (int)$image_type['height']*2))
					{
						$this->logError(sprintf("Impossible de générer le thumbnail %s", $dst));
						$res = false;
					}
				}
			}

			/* Ménage dans les tmp/mini */
			foreach (glob(_PS_TMP_IMG_DIR_.$miniName.'_'.$objectId.'_*.*', GLOB_NOSORT) as $file)
				@unlink($file);
			foreach (glob(_PS_TMP_IMG_DIR_.$miniName.'_mini_'.$objectId.'_*.*', GLOB_NOSORT) as $file)
				@unlink($file);

			return $res;
		}
		
		protected function createPSImg($path, $polarisPhotoId, $psObj)
		{						
			$polarisImgPath = PolarisLinkAPI::GetPolarisPhotoPathFor($this->Link->LinkID, $polarisPhotoId);
			
			if (file_exists($polarisImgPath))
			{				
				if (file_exists($path) && !@unlink ($path))
				{
					$this->logError(sprintf("impossible de supprimer le fichier %s (nettoyage avant création photo)", $path));
					return false;
				}

				$infoFile = $path.".polarislink";
				if (file_exists($infoFile) && !@unlink ($infoFile))
				{
					$this->logError(sprintf("impossible de supprimer le fichier %s (nettoyage avant création photo)", $infoFile));
					return false;
				}

				if ($this->Link->OPT_NO_SYMLNK_IMG)
				{
					// Soit nous utilisons celle de la copie
					// prends de la place mais fonctionne partout
					if (@copy($polarisImgPath, $path))
					{
						// On se laisse un repère !
						if (!file_put_contents($infoFile, $polarisPhotoId))
						{
							$this->logError(sprintf("impossible de sauver les informations photo %s", $infoFile));
							return NULL;
						}
						return true;
					}
					else
						$this->logError(sprintf("impossible de copier la photo Polaris %s -> %s pour %s #%s n°%s !", 
							$polarisImgPath, $path,
							$polarisPhotoId, get_class($psObj), $psObj->id));
				}
				else
				{
					// Soit nous utilisons la méthode des symlink
					// plus économique et par défaut
					$relativePath = PolarisLinkAPI::getRelativePath($path, $polarisImgPath);
					if (@symlink($relativePath, $path))
					{
						return true;
					}
					else
						$this->logError(sprintf("impossible de lier la photo Polaris %s -> %s n°%s pour %s #%s : symlink impossible !", 
							$path, $relativePath,
							$polarisPhotoId, get_class($psObj), $psObj->id));
				}
			}
			else
				$this->logError(sprintf("photo Polaris n°%s introuvable sur le disque pour %s #%s", $polarisPhotoId, get_class($psObj), $psObj->id));

			return false;
		}

		/*
			Renvoi vrai si le Hash proposé pour la photo est le bon, faux autrement
		*/
		protected function CheckImageHash($polarisPhotoId, $hash)
		{
			if ($hash == null)
				return true;

			// TODO ?
			
			return true;
		}

		/*
			Importe un logo 
		*/
		protected function ImportPhotoLogo($hint, $photosPath, $title, $itemName, $psObj, $listPhotoPolaris, $hashToCheck = null)
		{
			if ($photo = $this->GetReferencesFromList($listPhotoPolaris, true))
			{
				// Windows ?
				if (DIRECTORY_SEPARATOR != '/')
					$photosPath = strtr($photosPath, '/', DIRECTORY_SEPARATOR);
				
				// On lit la photo actuelle
				$filename_final = $photosPath.($psObj->id).".jpg";				
				if ($this->GetPolarisPhotoFor($filename_final) != $photo || !$this->CheckImageHash($photo, $hashToCheck))
				{
					// L'image a changé
					$this->Log($psObj, "Modification des photos");
					$psObj->deleteImage();

					// nettoyage webp
					@unlink($photosPath.($psObj->id).".webp");
					foreach (glob($photosPath.($psObj->id)."-*.webp") as $file)
						@unlink($file);
					
					// dans les tmp
					@unlink(_PS_TMP_IMG_DIR_. $itemName . '_' . $psObj->id . '.webp');
					@unlink(_PS_TMP_IMG_DIR_. $itemName . '_mini_' . $psObj->id . '.webp');

					if ($photo > 0)
					{
						if ($this->createPSImg($filename_final, $photo, $psObj))
							$this->RebuildThumbnails($title, $itemName, $photosPath, $psObj->id, $psObj->id, "jpg");	
					}					
				}				
			}
		}		
		
		/*
			Initialisation, on se note un certain nombre de paramètres
		*/
		public function __construct()
		{			
		}
		
		/*
			Initialise l'instance
		*/		
		public function Init(PolarisLinkJobManager $jobManager, PolarisLinkSite $link, PolarisLinkRefCache $refCache = null)
		{
			$this->DB = Db::getInstance();
			$this->Link = $link;
			$this->JobManager = $jobManager;
			$this->CacheManager = new PolarisLinkCacheManager();
			$this->RefsCache = $refCache;
			$this->OnInit();
		}
		
		/*
			Méthode appelée à l'initialisation
		*/
		protected function OnInit()
		{
		}
	
		/*
			Méthode appelé sur un élément à importer
		*/
		protected function OnImportData($data)
		{
		}		

		/*
			Obtient la référence d'un XML
		*/
		protected function getReferenceFromData($data)
		{
			$ref = null;
			
			if (empty($ref) && isset($data["ref"])) $ref = $data["ref"];
			if (empty($ref) && isset($data["sku"])) $ref = $data["sku"];
			if (empty($ref)) $ref = $data->reference;

			return $ref;
		}		

		/*
			Méthode appelé pour importer un élément
		*/
		public function import(XMLReader $docxml, PolarisLinkJobs $job)
		{
			try
			{
				$this->Job = $job;
				$strData = $docxml->readOuterXML();
				if ($this->DebugMode)
				{
					echo "Import : ";
					try
					{
						$data = new \SimpleXMLElement($strData);
						
						$doc = new DomDocument();
						$doc->loadXML($data->asXML());
						$doc->preserveWhiteSpace = false;
						$doc->formatOutput = true;

						echo $doc->saveXML();						
					}
					catch (Exception $ex)
					{
						var_dump($strData);
					}
					echo "\n";
				}
				else
					$data = new \SimpleXMLElement($strData);
				
				$this->OnImportData($data);			
			}
			catch (PrestaShopDatabaseException $pex)
			{
				// Erreur fatale
				throw $pex;
			}
			catch (Exception $ex)
			{
				$ref = "";
				if (!empty($data))
					$ref = $this->getReferenceFromData($data);

				if (empty($ref))
					$ref = "?";
				
				throw new Exception(sprintf("%s, ref. %s", $docxml->name, $ref), 0, $ex);
			}
		}
	}
	